@@ -13,18 +13,7 @@ from jobflow.core.reference import OnMissing, OutputReference
 from jobflow.utils.uuid import suuid
 
 if typing.TYPE_CHECKING:
-    from typing import (
-        Any,
-        Callable,
-        Dict,
-        Hashable,
-        List,
-        Optional,
-        Set,
-        Tuple,
-        Type,
-        Union,
-    )
+    from typing import Any, Callable, Hashable
 
     from networkx import DiGraph
     from pydantic import BaseModel
@@ -57,6 +46,11 @@ class JobConfig(MSONable):
     pass_manager_config
         Whether to pass the manager configuration on to detour, addition, and
         replacement jobs.
+    response_manager_config
+        The custom configuration to pass to a detour, addition, or replacement job.
+        Using this kwarg will automatically take precedence over the behavior of
+        ``pass_manager_config`` such that a different configuration than
+        ``manger_config`` can be passed to downstream jobs.
 
     Returns
     -------
@@ -69,14 +63,15 @@ class JobConfig(MSONable):
     manager_config: dict = field(default_factory=dict)
     expose_store: bool = False
     pass_manager_config: bool = True
+    response_manager_config: dict = field(default_factory=dict)
 
 
-def job(method: Optional[Callable] = None, **job_kwargs):
+def job(method: Callable | None = None, **job_kwargs):
     """
     Wrap a function to produce a :obj:`Job`.
 
     :obj:`Job` objects are delayed function calls that can be used in an
-    :obj:`Flow`. A job is a composed of the function name and source and any
+    :obj:`Flow`. A job is composed of the function name and source and any
     arguments for the function. This decorator makes it simple to create
     job objects directly from a function definition. See the examples for more details.
 
@@ -121,9 +116,9 @@ def job(method: Optional[Callable] = None, **job_kwargs):
     OutputReference('abeb6f48-9b34-4698-ab69-e4dc2127ebe9')
 
     .. Note::
-        Because the task has not yet been run, the output value is :obj:`OutputReference`
-        object. References are automatically converted to their computed values
-        (resolved) when the task runs.
+        Because the task has not yet been run, the output value is an
+        :obj:`OutputReference` object. References are automatically converted to their
+        computed values (resolved) when the task runs.
 
     If a dictionary of values is returned, the values can be indexed in the usual
     way.
@@ -140,7 +135,7 @@ def job(method: Optional[Callable] = None, **job_kwargs):
         doesn't exist, this error will only be raised when the Job is executed.
 
     Jobs can return :obj:`.Response` objects that control the flow execution flow.
-    For example, to replace the current jub with another job, ``replace`` can be used.
+    For example, to replace the current job with another job, ``replace`` can be used.
 
     >>> from jobflow import Response
     >>> @job
@@ -254,8 +249,16 @@ class Job(MSONable):
         A dictionary of information that will get stored alongside the job output.
     config
         The config setting for the job.
-    host
-        The UUID of the host flow.
+    hosts
+        The list of UUIDs of the hosts containing the job. The object identified by one
+        UUID of the list should be contained in objects identified by its subsequent
+        elements.
+    metadata_updates
+        A list of updates for the metadata that will be applied to any Flow/Job
+        generated by the job.
+    config_updates
+        A list of updates for the config that will be applied to any Flow/Job generated
+        by the job.
     **kwargs
         Additional keyword arguments that can be used to specify which outputs to save
         in additional stores. The argument name gives the additional store name and the
@@ -304,22 +307,24 @@ class Job(MSONable):
     def __init__(
         self,
         function: Callable,
-        function_args: Tuple[Any, ...] = None,
-        function_kwargs: Dict[str, Any] = None,
-        output_schema: Optional[Type[BaseModel]] = None,
+        function_args: tuple[Any, ...] = None,
+        function_kwargs: dict[str, Any] = None,
+        output_schema: type[BaseModel] | None = None,
         uuid: str = None,
         index: int = 1,
-        name: Optional[str] = None,
-        metadata: Dict[str, Any] = None,
+        name: str | None = None,
+        metadata: dict[str, Any] = None,
         config: JobConfig = None,
-        host: Optional[str] = None,
+        hosts: list[str] | None = None,
+        metadata_updates: list[dict[str, Any]] | None = None,
+        config_updates: list[dict[str, Any]] | None = None,
         **kwargs,
     ):
         from copy import deepcopy
 
         from jobflow.utils.find import contains_flow_or_job
 
-        function_args = tuple() if function_args is None else function_args
+        function_args = () if function_args is None else function_args
         function_kwargs = {} if function_kwargs is None else function_kwargs
         uuid = suuid() if uuid is None else uuid
         metadata = {} if metadata is None else metadata
@@ -335,10 +340,12 @@ class Job(MSONable):
         self.name = name
         self.metadata = metadata
         self.config = config
-        self.host = host
+        self.hosts = hosts or []
+        self.metadata_updates = metadata_updates or []
+        self.config_updates = config_updates or []
         self._kwargs = kwargs
 
-        if sum([v is True for v in kwargs.values()]) > 1:
+        if sum(v is True for v in kwargs.values()) > 1:
             raise ValueError("Cannot select True for multiple additional stores.")
 
         if self.name is None:
@@ -361,7 +368,7 @@ class Job(MSONable):
             )
 
     @property
-    def input_references(self) -> Tuple[jobflow.OutputReference, ...]:
+    def input_references(self) -> tuple[jobflow.OutputReference, ...]:
         """
         Find :obj:`.OutputReference` objects in the job inputs.
 
@@ -372,14 +379,14 @@ class Job(MSONable):
         """
         from jobflow.core.reference import find_and_get_references
 
-        references: Set[jobflow.OutputReference] = set()
+        references: set[jobflow.OutputReference] = set()
         for arg in tuple(self.function_args) + tuple(self.function_kwargs.values()):
             references.update(find_and_get_references(arg))
 
         return tuple(references)
 
     @property
-    def input_uuids(self) -> Tuple[str, ...]:
+    def input_uuids(self) -> tuple[str, ...]:
         """
         Uuids of any :obj:`.OutputReference` objects in the job inputs.
 
@@ -388,10 +395,10 @@ class Job(MSONable):
         tuple(str, ...)
            The UUIDs of the references in the job inputs.
         """
-        return tuple([ref.uuid for ref in self.input_references])
+        return tuple(ref.uuid for ref in self.input_references)
 
     @property
-    def input_references_grouped(self) -> Dict[str, Tuple[OutputReference, ...]]:
+    def input_references_grouped(self) -> dict[str, tuple[OutputReference, ...]]:
         """
         Group any :obj:`.OutputReference` objects in the job inputs by their UUIDs.
 
@@ -409,7 +416,7 @@ class Job(MSONable):
         return {k: tuple(v) for k, v in groups.items()}
 
     @property
-    def maker(self) -> Optional[jobflow.Maker]:
+    def maker(self) -> jobflow.Maker | None:
         """
         Get the host :obj:`.Maker` object if a job is to run a maker function.
 
@@ -439,7 +446,7 @@ class Job(MSONable):
 
         edges = []
         for uuid, refs in self.input_references_grouped.items():
-            properties: Union[List[str], str] = [
+            properties: list[str] | str = [
                 ref.attributes_formatted[-1]
                 .replace("[", "")
                 .replace("]", "")
@@ -456,6 +463,18 @@ class Job(MSONable):
         graph.add_edges_from(edges)
         return graph
 
+    @property
+    def host(self):
+        """
+        UUID of the first Flow that contains the Job.
+
+        Returns
+        -------
+        str
+            the UUID of the host.
+        """
+        return self.hosts[0] if self.hosts else None
+
     def set_uuid(self, uuid: str):
         """
         Set the UUID of the job.
@@ -501,6 +520,7 @@ class Job(MSONable):
         from datetime import datetime
 
         from jobflow import CURRENT_JOB
+        from jobflow.core.flow import get_flow
 
         index_str = f", {self.index}" if self.index != 1 else ""
         logger.info(f"Starting job - {self.name} ({self.uuid}{index_str})")
@@ -527,23 +547,52 @@ class Job(MSONable):
         if response.replace is not None:
             response.replace = prepare_replace(response.replace, self)
 
-        if self.config.pass_manager_config:
+        if response.addition is not None:
+            # wrap the detour in a Flow to avoid problems if it needs to get
+            # wrapped at a later stage
+            response.addition = get_flow(response.addition)
+
+        if response.detour is not None:
+            # wrap the detour in a Flow to avoid problems if it needs to get
+            # wrapped at a later stage
+            response.detour = get_flow(response.detour)
+
+        # common actions that should be applied to all newly generated Jobs/Flows
+        for new_jobs in (response.replace, response.addition, response.detour):
+            if new_jobs is not None:
+                new_jobs.add_hosts_uuids(self.hosts)
+                for metadata_update in self.metadata_updates:
+                    new_jobs.update_metadata(**metadata_update, dynamic=True)
+                for config_update in self.config_updates:
+                    new_jobs.update_config(**config_update, dynamic=True)
+
+        if self.config.response_manager_config:
+            passed_config = self.config.response_manager_config
+        elif self.config.pass_manager_config:
+            passed_config = self.config.manager_config
+        else:
+            passed_config = None
+
+        if passed_config:
+
             if response.addition is not None:
-                pass_manager_config(response.addition, self.config.manager_config)
+                pass_manager_config(response.addition, passed_config)
 
             if response.detour is not None:
-                pass_manager_config(response.detour, self.config.manager_config)
+                pass_manager_config(response.detour, passed_config)
 
             if response.replace is not None:
-                pass_manager_config(response.replace, self.config.manager_config)
+                pass_manager_config(response.replace, passed_config)
 
         try:
-            output = jsanitize(response.output, strict=True, enum_values=True)
-        except AttributeError:
+            output = jsanitize(
+                response.output, strict=True, enum_values=True, allow_bson=True
+            )
+        except AttributeError as err:
             raise RuntimeError(
                 "Job output contained an object that is not MSONable and therefore "
                 "could not be serialized."
-            )
+            ) from err
 
         save = {k: "output" if v is True else v for k, v in self._kwargs.items()}
         data = {
@@ -552,6 +601,8 @@ class Job(MSONable):
             "output": output,
             "completed_at": datetime.now().isoformat(),
             "metadata": self.metadata,
+            "hosts": self.hosts,
+            "name": self.name,
         }
         store.update(data, key=["uuid", "index"], save=save)
 
@@ -585,7 +636,7 @@ class Job(MSONable):
 
         from jobflow.core.reference import find_and_resolve_references
 
-        cache: Dict[str, Any] = {}
+        cache: dict[str, Any] = {}
         resolved_args = find_and_resolve_references(
             self.function_args,
             store,
@@ -612,9 +663,9 @@ class Job(MSONable):
 
     def update_kwargs(
         self,
-        update: Dict[str, Any],
-        name_filter: Optional[str] = None,
-        function_filter: Optional[Callable] = None,
+        update: dict[str, Any],
+        name_filter: str | None = None,
+        function_filter: Callable | None = None,
         dict_mod: bool = False,
     ):
         """
@@ -625,18 +676,20 @@ class Job(MSONable):
         update
             The updates to apply.
         name_filter
-            A filter for the job name.
+            A filter for the job name. Only jobs with a matching name will be updated.
+            Includes partial matches, e.g. "ad" will match a job with the name "adder".
         function_filter
-            Only filter matching functions.
+            A filter for the job function. Only jobs with a matching function will be
+            updated.
         dict_mod
             Use the dict mod language to apply updates. See :obj:`.DictMods` for more
             details.
 
         Examples
         --------
-        Consider a flow containing a simple job with a ``number`` keyword argument.
+        Consider a simple job with a ``number`` keyword argument.
 
-        >>> from jobflow import job, Flow
+        >>> from jobflow import job
         >>> @job
         ... def add(a, number=5):
         ...     return a + number
@@ -666,9 +719,9 @@ class Job(MSONable):
 
     def update_maker_kwargs(
         self,
-        update: Dict[str, Any],
-        name_filter: Optional[str] = None,
-        class_filter: Optional[Type[jobflow.Maker]] = None,
+        update: dict[str, Any],
+        name_filter: str | None = None,
+        class_filter: type[jobflow.Maker] | None = None,
         nested: bool = True,
         dict_mod: bool = False,
     ):
@@ -680,10 +733,12 @@ class Job(MSONable):
         update
             The updates to apply.
         name_filter
-            A filter for the Maker name.
+            A filter for the Maker name. Only Makers with a matching name will be
+            updated. Includes partial matches, e.g. "ad" will match a Maker with the
+            name "adder".
         class_filter
-            A filter for the maker class. Note the class filter will match any
-            subclasses.
+            A filter for the maker class. Only Makers with a matching class will be
+            updated. Note the class filter will match any subclasses.
         nested
             Whether to apply the updates to Maker objects that are themselves kwargs
             of Maker, job, or flow objects. See examples for more details.
@@ -732,13 +787,13 @@ class Job(MSONable):
         The following update will apply to the nested ``AddMaker`` in the kwargs of the
         ``RestartMaker``:
 
-        >>> my_job.update_maker_kwargs({"number": 10}, function_filter=AddMaker)
+        >>> my_job.update_maker_kwargs({"number": 10}, class_filter=AddMaker)
 
         However, if ``nested=False``, then the update will not be applied to the nested
         Maker:
 
         >>> my_job.update_maker_kwargs(
-        ...     {"number": 10}, function_filter=AddMaker, nested=False
+        ...     {"number": 10}, class_filter=AddMaker, nested=False
         ... )
         """
         from jobflow import Maker
@@ -751,7 +806,7 @@ class Job(MSONable):
                 nested=nested,
                 dict_mod=dict_mod,
             )
-            setattr(self, "function", getattr(maker, self.function.__name__))
+            self.function = getattr(maker, self.function.__name__)
         elif nested:
             # also look for makers in job args and kwargs
             new_args = []
@@ -795,7 +850,219 @@ class Job(MSONable):
         else:
             self.name += append_str
 
-    def as_dict(self) -> Dict:
+    def update_metadata(
+        self,
+        update: dict[str, Any],
+        name_filter: str | None = None,
+        function_filter: Callable | None = None,
+        dict_mod: bool = False,
+        dynamic: bool = True,
+    ):
+        """
+        Update the metadata of the job.
+
+        Can optionally apply the same updates at runtime to any Job or Flow generated
+        by this job.
+
+        Parameters
+        ----------
+        update
+            The updates to apply.
+        name_filter
+            A filter for the job name. Only jobs with a matching name will be updated.
+            Includes partial matches, e.g. "ad" will match a job with the name "adder".
+        function_filter
+            A filter for the job function. Only jobs with a matching function will be
+            updated.
+        dict_mod
+            Use the dict mod language to apply updates. See :obj:`.DictMods` for more
+            details.
+        dynamic
+            The updates will be propagated to Jobs/Flows dynamically generated at
+            runtime.
+
+        Examples
+        --------
+        Consider a simple job that makes use of a :obj:`Maker` to generate additional
+        jobs at runtime (see :obj:`Response` options for more details):
+
+        >>> @job
+        ... def use_maker(maker):
+        ...     return Response(replace=maker.make())
+
+        Calling `update_metadata` with `dynamic` set to `True` (the default)
+
+        >>> test_job = use_maker(ExampleMaker())
+        ... test_job.update_metadata({"example": 1}, dynamic=True)
+
+        will not only set the `example` metadata to the `test_job`, but also to all the
+        new Jobs that will be generated at runtime by the ExampleMaker.
+
+        `update_metadata` can be called multiple times with different `name_filter` or
+        `function_filter` to control which Jobs will be updated.
+
+        At variance, if `dynamic` is set to `False` the `example` metadata will only be
+        added to the `test_job` and not to the generated Jobs.
+        """
+        from jobflow.utils.dict_mods import apply_mod
+
+        if dynamic:
+            dict_input = {
+                "update": update,
+                "name_filter": name_filter,
+                "function_filter": function_filter,
+                "dict_mod": dict_mod,
+            }
+            self.metadata_updates.append(dict_input)
+
+        # unwrap the functions in case the job is a decorated one
+        function_filter = getattr(function_filter, "__wrapped__", function_filter)
+        function = getattr(self.function, "__wrapped__", self.function)
+
+        # if function_filter is not None and function_filter != self.function:
+        if function_filter is not None and function_filter != function:
+            return
+
+        if name_filter is not None and (
+            self.name is None or name_filter not in self.name
+        ):
+            return
+
+        # if we get to here then we pass all the filters
+        if dict_mod:
+            apply_mod(update, self.metadata)
+        else:
+            self.metadata.update(update)
+
+    def update_config(
+        self,
+        config: JobConfig | dict,
+        name_filter: str = None,
+        function_filter: Callable = None,
+        attributes: list[str] | str = None,
+        dynamic: bool = True,
+    ):
+        """
+        Update the job config.
+
+        Can optionally apply the same updates at runtime to any Job or Flow generated
+        by this job.
+
+        Parameters
+        ----------
+        config
+            A JobConfig object or a dict with containing the attributes to update.
+        name_filter
+            A filter for the job name. Only jobs with a matching name will be updated.
+            Includes partial matches, e.g. "ad" will match a job with the name "adder".
+        function_filter
+            A filter for the job function. Only jobs with a matching function will be
+            updated.
+        attributes :
+            Which attributes of the job config to set. Can be specified as one or more
+            attributes specified by their name.
+        dynamic
+            The updates will be propagated to Jobs/Flows dynamically generated at
+            runtime.
+
+        Examples
+        --------
+        Consider a simple job.
+
+        >>> from jobflow import job, JobConfig
+        >>> @job
+        ... def add(a, b):
+        ...     return a + b
+        >>> add_job = add(1, 2)
+
+        The ``config`` can be updated using.
+
+        >>> new_config = JobConfig(
+        ...    manager_config={"_fworker": "myfworker"}, resolve_references=False
+        ... )
+        >>> add_job.update_config(new_config)
+
+        To only update specific attributes, the ``attributes`` argument can be
+        specified. For example, the following will only update the "manager_config"
+        attribute of the job config.
+
+        >>> add_job.update_config(new_config, attributes="manager_config")
+
+        Alternatively, the config can be specified as a dictionary with keys that are
+        attributes of the JobConfig object. This allows you to specify updates without
+        having to create a completely new JobConfig object. For example:
+
+        >>> add_job.update_config({"manager_config": {"_fworker": "myfworker"}})
+
+        Consider instead a simple job that makes use of a :obj:`Maker` to generate
+        additional jobs at runtime (see :obj:`Response` options for more details):
+
+        >>> @job
+        ... def use_maker(maker):
+        ...     return Response(replace=maker.make())
+
+        Calling `update_config` with `dynamic` set to `True` (the default)
+
+        >>> test_job = use_maker(ExampleMaker())
+        ... test_job.update_config({"manager_config": {"_fworker": "myfworker"}})
+
+        will not only set the `manager_config` to the `test_job`, but also to all the
+        new Jobs that will be generated at runtime by the ExampleMaker.
+
+        `update_config` can be called multiple times with different `name_filter` or
+        `function_filter` to control which Jobs will be updated.
+
+        At variance, if `dynamic` is set to `False` the `manager_config` option will
+        only be set for the `test_job` and not for the generated Jobs.
+        """
+        if dynamic:
+            dict_input = {
+                "config": config,
+                "name_filter": name_filter,
+                "function_filter": function_filter,
+                "attributes": attributes,
+            }
+            self.config_updates.append(dict_input)
+
+        # unwrap the functions in case the job is a decorated one
+        function_filter = getattr(function_filter, "__wrapped__", function_filter)
+        function = getattr(self.function, "__wrapped__", self.function)
+
+        # if function_filter is not None and function_filter != self.function:
+        if function_filter is not None and function_filter != function:
+            return
+
+        if name_filter is not None and (
+            self.name is None or name_filter not in self.name
+        ):
+            return
+
+        # if we get to here then we pass all the filters
+        if isinstance(config, dict):
+            # convert dict specification to a JobConfig but set the attributes
+            if attributes is None:
+                attributes = list(config.keys())
+
+            attributes = [attributes] if isinstance(attributes, str) else attributes
+            if not set(attributes).issubset(set(config.keys())):
+                raise ValueError(
+                    "Specified attributes include a key that is not present in the "
+                    "config dictionary."
+                )
+            config = JobConfig(**config)
+
+        if attributes is None:
+            # overwrite the whole config
+            self.config = config
+        else:
+            # only update the specified attributes
+            attributes = [attributes] if isinstance(attributes, str) else attributes
+            for attr in attributes:
+                if not hasattr(self.config, attr):
+                    raise ValueError(f"Unknown JobConfig attribute: {attr}")
+                setattr(self.config, attr, getattr(config, attr))
+
+    def as_dict(self) -> dict:
         """Serialize the job as a dictionary."""
         d = super().as_dict()
 
@@ -814,9 +1081,38 @@ class Job(MSONable):
         else:
             super().__setattr__(key, value)
 
+    def add_hosts_uuids(self, hosts_uuids: str | list[str], prepend: bool = False):
+        """
+        Add a list of UUIDs to the internal list of hosts.
+
+        The elements of the list are supposed to be ordered in such a way that
+        the object identified by one UUID of the list is contained in objects
+        identified by its subsequent elements.
+
+        Parameters
+        ----------
+        hosts_uuids
+            A list of UUIDs to add.
+        prepend
+            Insert the UUIDs at the beginning of the list rather than extending it.
+        """
+        if not isinstance(hosts_uuids, (list, tuple)):
+            hosts_uuids = [hosts_uuids]
+        if prepend:
+            self.hosts[0:0] = hosts_uuids
+        else:
+            self.hosts.extend(hosts_uuids)
+
+
+# For type checking, the Response output type can be specified
+# in a type hint via this type variable.
+# For example, a signature `-> Response[int]` would require
+# that the Response.output is an int.
+T = typing.TypeVar("T")
+
 
 @dataclass
-class Response:
+class Response(typing.Generic[T]):
     """
     The :obj:`Response` contains the output, detours, and stop commands of a job.
 
@@ -838,19 +1134,19 @@ class Response:
         Stop executing all remaining jobs.
     """
 
-    output: Optional[Any] = None
-    detour: Optional[Union[jobflow.Flow, Job, List[Job], List[jobflow.Flow]]] = None
-    addition: Optional[Union[jobflow.Flow, Job, List[Job], List[jobflow.Flow]]] = None
-    replace: Optional[Union[jobflow.Flow, Job, List[Job], List[jobflow.Flow]]] = None
-    stored_data: Optional[Dict[Hashable, Any]] = None
+    output: T | None = None
+    detour: jobflow.Flow | Job | list[Job] | list[jobflow.Flow] | None = None
+    addition: jobflow.Flow | Job | list[Job] | list[jobflow.Flow] | None = None
+    replace: jobflow.Flow | Job | list[Job] | list[jobflow.Flow] | None = None
+    stored_data: dict[Hashable, Any] | None = None
     stop_children: bool = False
     stop_jobflow: bool = False
 
     @classmethod
     def from_job_returns(
         cls,
-        job_returns: Optional[Any],
-        output_schema: Optional[Type[BaseModel]] = None,
+        job_returns: Any | None,
+        output_schema: type[BaseModel] | None = None,
     ) -> Response:
         """
         Generate a :obj:`Response` from the outputs of a :obj:`Job`.
@@ -894,7 +1190,7 @@ class Response:
         return cls(output=apply_schema(job_returns, output_schema))
 
 
-def apply_schema(output: Any, schema: Optional[Type[BaseModel]]):
+def apply_schema(output: Any, schema: type[BaseModel] | None):
     """
     Apply schema to job outputs.
 
@@ -949,9 +1245,9 @@ def store_inputs(inputs: Any) -> Any:
 
 
 def prepare_replace(
-    replace: Union[jobflow.Flow, Job, List[Job]],
+    replace: jobflow.Flow | Job | list[Job],
     current_job: Job,
-) -> Union[jobflow.Flow, Job, List[Job]]:
+) -> jobflow.Flow:
     """
     Prepare a replacement :obj:`Flow` or :obj:`Job`.
 
@@ -970,8 +1266,8 @@ def prepare_replace(
 
     Returns
     -------
-    Flow or Job
-        The updated flow or job.
+    Flow
+        The updated flow.
     """
     from jobflow.core.flow import Flow
 
@@ -987,7 +1283,7 @@ def prepare_replace(
         store_output_job.index = current_job.index + 1
         store_output_job.metadata = current_job.metadata
         store_output_job.output_schema = current_job.output_schema
-        replace.jobs.append(store_output_job)
+        replace.add_jobs(store_output_job)
 
     elif isinstance(replace, Job):
         # replace is a single Job
@@ -1001,12 +1297,14 @@ def prepare_replace(
         if not replace.output_schema:
             replace.output_schema = current_job.output_schema
 
+        replace = Flow(jobs=replace, output=replace.output)
+
     return replace
 
 
 def pass_manager_config(
-    jobs: Union[Job, jobflow.Flow, List[Union[Job, jobflow.Flow]]],
-    manager_config: Dict[str, Any],
+    jobs: Job | jobflow.Flow | list[Job | jobflow.Flow],
+    manager_config: dict[str, Any],
 ):
     """
     Pass the manager config on to any jobs in the jobs array.
@@ -1022,7 +1320,7 @@ def pass_manager_config(
     """
     from copy import deepcopy
 
-    all_jobs: List[Job] = []
+    all_jobs: list[Job] = []
 
     def get_jobs(arg):
 
