@@ -2,17 +2,18 @@
 
 from __future__ import annotations
 
+import contextlib
 import typing
-from typing import Any, Dict, Optional, Sequence, Tuple, Type
+from typing import Any, Sequence
 
 from monty.json import MontyDecoder, MontyEncoder, MSONable, jsanitize
+from pydantic import BaseModel
+from pydantic.utils import lenient_issubclass
 
 from jobflow.utils.enum import ValueEnum
 
 if typing.TYPE_CHECKING:
 
-    from pydantic import BaseModel
-
     import jobflow
 
 __all__ = [
@@ -93,8 +94,8 @@ class OutputReference(MSONable):
     def __init__(
         self,
         uuid: str,
-        attributes: Tuple[Tuple[str, Any], ...] = tuple(),
-        output_schema: Optional[Type[BaseModel]] = None,
+        attributes: tuple[tuple[str, Any], ...] = (),
+        output_schema: type[BaseModel] | None = None,
     ):
         super().__init__()
         self.uuid = uuid
@@ -109,8 +110,8 @@ class OutputReference(MSONable):
 
     def resolve(
         self,
-        store: Optional[jobflow.JobStore],
-        cache: Optional[Dict[str, Any]] = None,
+        store: jobflow.JobStore | None,
+        cache: dict[str, Any] | None = None,
         on_missing: OnMissing = OnMissing.ERROR,
     ) -> Any:
         """
@@ -157,12 +158,10 @@ class OutputReference(MSONable):
         index = None if result is None else result["index"]
 
         if index is not None and index not in cache[self.uuid]:
-            try:
+            with contextlib.suppress(ValueError):
                 cache[self.uuid][index] = store.get_output(
                     self.uuid, which="last", load=True, on_missing=on_missing
                 )
-            except ValueError:
-                pass
 
         if on_missing == OnMissing.ERROR and index not in cache[self.uuid]:
             istr = f" ({index})" if index is not None else ""
@@ -183,12 +182,8 @@ class OutputReference(MSONable):
         cache[self.uuid][index] = data
 
         for attr_type, attr in self.attributes:
-            if attr_type == "i":
-                # index
-                data = data[attr]
-            else:
-                # attribute access
-                data = getattr(data, attr)
+            # i means index else use attribute access
+            data = data[attr] if attr_type == "i" else getattr(data, attr)
 
         return data
 
@@ -221,10 +216,13 @@ class OutputReference(MSONable):
 
     def __getitem__(self, item) -> OutputReference:
         """Index the reference."""
+        subschema = None
         if self.output_schema is not None:
-            validate_schema_access(self.output_schema, item)
+            _, subschema = validate_schema_access(self.output_schema, item)
 
-        return OutputReference(self.uuid, self.attributes + (("i", item),))
+        return OutputReference(
+            self.uuid, (*self.attributes, ("i", item)), output_schema=subschema
+        )
 
     def __getattr__(self, item) -> OutputReference:
         """Attribute access of the reference."""
@@ -234,10 +232,13 @@ class OutputReference(MSONable):
             # This is necessary to trick monty/pydantic.
             raise AttributeError
 
+        subschema = None
         if self.output_schema is not None:
-            validate_schema_access(self.output_schema, item)
+            _, subschema = validate_schema_access(self.output_schema, item)
 
-        return OutputReference(self.uuid, self.attributes + (("a", item),))
+        return OutputReference(
+            self.uuid, (*self.attributes, ("a", item)), output_schema=subschema
+        )
 
     def __setattr__(self, attr, val):
         """Set attribute."""
@@ -247,6 +248,10 @@ class OutputReference(MSONable):
         else:
             raise TypeError("OutputReference objects are immutable")
 
+    def __iter__(self):
+        """Make sure OutputReference is not iterable."""
+        raise TypeError("OutputReference objects are not iterable")
+
     def __setitem__(self, index, val):
         """Set item."""
         # Setting items via indexing is not allowed.
@@ -305,9 +310,9 @@ class OutputReference(MSONable):
 def resolve_references(
     references: Sequence[OutputReference],
     store: jobflow.JobStore,
-    cache: Optional[Dict[str, Any]] = None,
+    cache: dict[str, Any] | None = None,
     on_missing: OnMissing = OnMissing.ERROR,
-) -> Dict[OutputReference, Any]:
+) -> dict[OutputReference, Any]:
     """
     Resolve multiple output references.
 
@@ -357,7 +362,7 @@ def resolve_references(
     return resolved_references
 
 
-def find_and_get_references(arg: Any) -> Tuple[OutputReference, ...]:
+def find_and_get_references(arg: Any) -> tuple[OutputReference, ...]:
     """
     Find and extract output references.
 
@@ -380,25 +385,25 @@ def find_and_get_references(arg: Any) -> Tuple[OutputReference, ...]:
 
     if isinstance(arg, OutputReference):
         # if the argument is a reference then stop there
-        return tuple([arg])
+        return (arg,)
 
     elif isinstance(arg, (float, int, str, bool)):
         # argument is a primitive, we won't find a reference here
-        return tuple()
+        return ()
 
-    arg = jsanitize(arg, strict=True, enum_values=True)
+    arg = jsanitize(arg, strict=True, enum_values=True, allow_bson=True)
 
     # recursively find any reference classes
     locations = find_key_value(arg, "@class", "OutputReference")
 
     # deserialize references and return
-    return tuple([OutputReference.from_dict(get(arg, loc)) for loc in locations])
+    return tuple(OutputReference.from_dict(get(arg, loc)) for loc in locations)
 
 
 def find_and_resolve_references(
     arg: Any,
     store: jobflow.JobStore,
-    cache: Optional[Dict[str, Any]] = None,
+    cache: dict[str, Any] | None = None,
     on_missing: OnMissing = OnMissing.ERROR,
 ) -> Any:
     """
@@ -443,7 +448,7 @@ def find_and_resolve_references(
         return arg
 
     # serialize the argument to a dictionary
-    encoded_arg = jsanitize(arg, strict=True, enum_values=True)
+    encoded_arg = jsanitize(arg, strict=True, enum_values=True, allow_bson=True)
 
     # recursively find any reference classes
     locations = find_key_value(encoded_arg, "@class", "OutputReference")
@@ -472,10 +477,14 @@ def find_and_resolve_references(
     return MontyDecoder().process_decoded(encoded_arg)
 
 
-def validate_schema_access(schema: Type[BaseModel], item: str):
+def validate_schema_access(
+    schema: type[BaseModel], item: str
+) -> tuple[bool, BaseModel | None]:
     """
     Validate that an attribute or index access is supported by a model.
 
+    If the item is associated to a nested model the class is returned.
+
     Parameters
     ----------
     schema
@@ -490,10 +499,17 @@ def validate_schema_access(schema: Type[BaseModel], item: str):
 
     Returns
     -------
-    bool
-        Returns ``True`` if the schema access was valid.
+    tuple[bool, BaseModel]
+        the bool is ``True`` if the schema access was valid.
+        The BaseModel class associated with the item, if any.
     """
     schema_dict = schema.schema()
     if item not in schema_dict["properties"]:
         raise AttributeError(f"{schema.__name__} does not have attribute '{item}'.")
-    return True
+
+    subschema = None
+    item_type = schema.__fields__[item].outer_type_
+    if lenient_issubclass(item_type, BaseModel):
+        subschema = item_type
+
+    return True, subschema
