@@ -15,7 +15,7 @@ import typing
 from monty.design_patterns import singleton
 
 if typing.TYPE_CHECKING:
-    from typing import Optional
+    pass
 
     import jobflow
 
@@ -24,10 +24,13 @@ __all__ = ["CURRENT_JOB"]
 
 @singleton
 class State:
-    job: Optional[jobflow.Job] = None
-    store: Optional[jobflow.JobStore] = None
+    """State of the current job and store."""
+
+    job: jobflow.Job | None = None
+    store: jobflow.JobStore | None = None
 
     def reset(self):
+        """Reset the current state."""
         self.job = None
         self.store = None
 
