@@ -12,7 +12,7 @@ from jobflow.core.reference import OnMissing
 if typing.TYPE_CHECKING:
     from enum import Enum
     from pathlib import Path
-    from typing import Any, Dict, Iterator, List, Optional, Tuple, Type, Union
+    from typing import Any, Dict, Iterator, List, Optional, Type, Union
 
     from maggma.core import Sort
 
@@ -50,11 +50,10 @@ class JobStore(Store):
     def __init__(
         self,
         docs_store: Store,
-        additional_stores: Optional[Dict[str, Store]] = None,
+        additional_stores: dict[str, Store] | None = None,
         save: save_type = None,
         load: load_type = False,
     ):
-        """ """
         self.docs_store = docs_store
         if additional_stores is None:
             self.additional_stores = {}
@@ -64,7 +63,9 @@ class JobStore(Store):
         # enforce uuid key
         self.docs_store.key = "uuid"
         for additional_store in self.additional_stores.values():
-            additional_store.key = "blob_id"
+            additional_store.key = "blob_uuid"
+            if hasattr(additional_store, "searchable_fields"):
+                additional_store.searchable_fields.extend(["job_uuid", "job_index"])
 
         if save is None or save is False:
             save = {}
@@ -115,7 +116,7 @@ class JobStore(Store):
         for additional_store in self.additional_stores.values():
             additional_store.close()
 
-    def count(self, criteria: Optional[Dict] = None) -> int:
+    def count(self, criteria: dict | None = None) -> int:
         """
         Count the number of documents matching the query criteria.
 
@@ -133,13 +134,13 @@ class JobStore(Store):
 
     def query(
         self,
-        criteria: Optional[Dict] = None,
-        properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Union[Sort, int]]] = None,
+        criteria: dict | None = None,
+        properties: dict | list | None = None,
+        sort: dict[str, Sort | int] | None = None,
         skip: int = 0,
         limit: int = 0,
         load: load_type = None,
-    ) -> Iterator[Dict]:
+    ) -> Iterator[dict]:
         """
         Query the JobStore for documents.
 
@@ -189,7 +190,7 @@ class JobStore(Store):
                 # 1. Find the locations of all blob identifiers.
                 # 2. Filter the locations based on the load criteria.
                 # 3. Resolve all data blobs using the data store.
-                # 4. Insert the data blobs into the document
+                # 4. Insert the data blobs into the document.
                 locations = find_key(doc, "blob_uuid")
                 all_blobs = [get(doc, list(loc)) for loc in locations]
                 grouped_blobs = _filter_blobs(all_blobs, locations, load_keys)
@@ -209,7 +210,7 @@ class JobStore(Store):
                         properties=["blob_uuid", "data"],
                     )
                     object_map = {o["blob_uuid"]: o["data"] for o in objects}
-                    inserts = {tuple(l): object_map[o] for o, l in object_info.items()}
+                    inserts = {tuple(v): object_map[k] for k, v in object_info.items()}
                     to_insert.update(inserts)
 
                 update_in_dictionary(doc, to_insert)
@@ -218,11 +219,11 @@ class JobStore(Store):
 
     def query_one(
         self,
-        criteria: Optional[Dict] = None,
-        properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Union[Sort, int]]] = None,
+        criteria: dict | None = None,
+        properties: dict | list | None = None,
+        sort: dict[str, Sort | int] | None = None,
         load: load_type = None,
-    ) -> Optional[Dict]:
+    ) -> dict | None:
         """
         Query the Store for a single document.
 
@@ -252,9 +253,9 @@ class JobStore(Store):
 
     def update(
         self,
-        docs: Union[List[Dict], Dict],
-        key: Union[List, str, None] = None,
-        save: Union[bool, save_type] = None,
+        docs: list[dict] | dict,
+        key: list | str | None = None,
+        save: bool | save_type = None,
     ):
         """
         Update or insert documents into the Store.
@@ -303,7 +304,9 @@ class JobStore(Store):
                         locations.extend(find_key(doc, save_key, include_end=True))
 
                     objects = [get(doc, list(loc)) for loc in locations]
-                    object_map = dict(zip(map(tuple, locations), objects))
+                    object_map = {
+                        tuple(k): o for k, o in zip(locations, objects) if o is not None
+                    }
                     object_info = {
                         k: _get_blob_info(o, store_name) for k, o in object_map.items()
                     }
@@ -325,7 +328,13 @@ class JobStore(Store):
         self.docs_store.update(dict_docs, key=key)
 
         for store_name, blobs in blob_data.items():
-            store = self.additional_stores.get(store_name, None)
+            # Here we use a try/except with a self.additional_stores[store_name]
+            # instead of self.additional_store.get(store_name, None) to allow
+            # for additional_stores to be a defaultdict with a MemoryStore's default.
+            try:
+                store = self.additional_stores[store_name]
+            except KeyError:
+                store = None
 
             if store is None:
                 raise ValueError(f"Unrecognised additional store name: {store_name}")
@@ -352,14 +361,14 @@ class JobStore(Store):
 
     def groupby(
         self,
-        keys: Union[List[str], str],
-        criteria: Optional[Dict] = None,
-        properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Union[Sort, int]]] = None,
+        keys: list[str] | str,
+        criteria: dict | None = None,
+        properties: dict | list | None = None,
+        sort: dict[str, Sort | int] | None = None,
         skip: int = 0,
         limit: int = 0,
         load: load_type = None,
-    ) -> Iterator[Tuple[Dict, List[Dict]]]:
+    ) -> Iterator[tuple[dict, list[dict]]]:
         """
         Group documents by keys.
 
@@ -413,12 +422,12 @@ class JobStore(Store):
             return tuple(get(doc, k) for k in keys)
 
         for vals, group in groupby(sorted(data, key=grouping_keys), key=grouping_keys):
-            doc: Dict[str, Any] = {}
+            doc: dict[str, Any] = {}
             for k, v in zip(keys, vals):
                 set_(doc, k, v)
             yield doc, list(group)
 
-    def remove_docs(self, criteria: Dict):
+    def remove_docs(self, criteria: dict):
         """
         Remove docs matching the criteria.
 
@@ -436,9 +445,9 @@ class JobStore(Store):
     def get_output(
         self,
         uuid: str,
-        which: Union[str, int] = "last",
+        which: str | int = "last",
         load: load_type = False,
-        cache: Optional[Dict[str, Any]] = None,
+        cache: dict[str, Any] | None = None,
         on_missing: OnMissing = OnMissing.ERROR,
     ):
         """
@@ -487,7 +496,7 @@ class JobStore(Store):
         if isinstance(which, int) or which in ("last", "first"):
             sort = -1 if which == "last" else 1
 
-            criteria: Dict[str, Any] = {"uuid": uuid}
+            criteria: dict[str, Any] = {"uuid": uuid}
             if isinstance(which, int):
                 # handle specific index specified
                 criteria["index"] = which
@@ -534,7 +543,7 @@ class JobStore(Store):
             )
 
     @classmethod
-    def from_file(cls: Type[T], db_file: Union[str, Path], **kwargs) -> T:
+    def from_file(cls: type[T], db_file: str | Path, **kwargs) -> T:
         """
         Create an JobStore from a database file.
 
@@ -594,13 +603,13 @@ class JobStore(Store):
         return cls.from_dict_spec(store_info, **kwargs)
 
     @classmethod
-    def from_dict_spec(cls, spec: dict, **kwargs) -> T:
+    def from_dict_spec(cls: type[T], spec: dict, **kwargs) -> T:
         """
         Create an JobStore from a dict specification.
 
         .. note::
             This function is different to ``JobStore.from_dict`` which is used to load
-            the monty serialised representation of the claas.
+            the monty serialised representation of the class.
 
         The dictionary should contain the keys "docs_store", "additional_stores" and
         any other keyword arguments supported by the :obj:`JobStore` constructor. The
@@ -651,28 +660,34 @@ class JobStore(Store):
         all_stores = {s.__name__: s for s in all_subclasses(maggma.stores.Store)}
 
         docs_store_info = spec["docs_store"]
-        docs_store_type = docs_store_info.pop("type")
-        docs_store = all_stores[docs_store_type](**docs_store_info)
+        docs_store = _construct_store(docs_store_info, all_stores)
 
         additional_stores = {}
         if "additional_stores" in spec:
             for store_name, info in spec["additional_stores"].items():
-                store_type = info.pop("type")
-                additional_stores[store_name] = all_stores[store_type](**info)
-
+                additional_stores[store_name] = _construct_store(info, all_stores)
         return cls(docs_store, additional_stores, **kwargs)
 
 
+def _construct_store(spec_dict, valid_stores):
+    """Parse the dict containing {"type": <StoreType>} recursively."""
+    store_type = spec_dict.pop("type")
+    for k, v in spec_dict.items():
+        if isinstance(v, dict) and "type" in v:
+            spec_dict[k] = _construct_store(v, valid_stores)
+    return valid_stores[store_type](**spec_dict)
+
+
 def _prepare_load(
     load: load_type,
-) -> Union[bool, Dict[str, Union[bool, List[Union[str, Tuple[str, str]]]]]]:
+) -> bool | dict[str, bool | list[str | tuple[str, str]]]:
     """Standardize load types."""
     from enum import Enum
 
     if isinstance(load, bool):
         return load
 
-    new_load: Dict[str, Union[bool, List[Union[str, Tuple[str, str]]]]] = {}
+    new_load: dict[str, bool | list[str | tuple[str, str]]] = {}
     for store_name, store_load in load.items():
         if isinstance(store_load, bool):
             new_load[store_name] = store_load
@@ -694,8 +709,8 @@ def _prepare_load(
 
 
 def _prepare_save(
-    save: Union[bool, save_type],
-) -> Dict[str, List[Union[str, Type[MSONable]]]]:
+    save: bool | save_type,
+) -> dict[str, list[str | type[MSONable]]]:
     """Standardize save type."""
     from enum import Enum
 
@@ -714,15 +729,15 @@ def _prepare_save(
 
 
 def _filter_blobs(
-    blob_infos: List[Dict],
-    locations: List[List[Any]],
-    load: Union[bool, Dict[str, Union[bool, List[Union[str, Tuple[str, str]]]]]] = None,
-) -> Dict[str, Tuple[List[Dict], List[List[Any]]]]:
+    blob_infos: list[dict],
+    locations: list[list[Any]],
+    load: bool | dict[str, bool | list[str | tuple[str, str]]] = None,
+) -> dict[str, tuple[list[dict], list[list[Any]]]]:
     """Filter and group blobs."""
     from collections import defaultdict
 
     def _group_blobs(infos, locs):
-        grouped = defaultdict(lambda: (list(), list()))
+        grouped = defaultdict(lambda: ([], []))
         for info, loc in zip(infos, locs):
             grouped[info["store"]][0].append(info)
             grouped[info["store"]][1].append(loc)
@@ -734,7 +749,7 @@ def _filter_blobs(
 
     new_blobs = []
     new_locations = []
-    for store_name, store_load in load.items():
+    for _store_name, store_load in load.items():
         for blob, location in zip(blob_infos, locations):
             if store_load is True:
                 new_blobs.append(blob)
@@ -748,17 +763,14 @@ def _filter_blobs(
                         isinstance(ltype, tuple)
                         and blob.get("@class", None) == ltype[1]
                         and blob.get("@module", None) == ltype[0]
-                    ):
-                        new_blobs.append(blob)
-                        new_locations.append(location)
-                    elif location[-1] == ltype:
+                    ) or location[-1] == ltype:
                         new_blobs.append(blob)
                         new_locations.append(location)
 
     return _group_blobs(new_blobs, new_locations)
 
 
-def _get_blob_info(obj: Any, store_name: str) -> Dict[str, str]:
+def _get_blob_info(obj: Any, store_name: str) -> dict[str, str]:
     from jobflow.utils.uuid import suuid
 
     class_name = ""
